#include <GL/glut.h>
#include <stdlib.h>
#include "Audio.h"

#include "GameSession.h"
#include "GameIntro.h"
#include "extern.h"
#include "utility.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif


char *message[3]={"press space to start game","a game by Paolo Cignoni","music by F. Del Priore"};
char *TitleMessage=message[0];


GameIntro::GameIntro()
{
  Title.Open("data/title.raw");
	Title.Flip();
}

/// This function clip the position of an object against the ViewFrustum
void GameIntro::Clip(Point3f &p)
  {
	 if(p.x() > 200-p.z() || p.y() > 200-p.z())
		{
		  p.x()=(drand48()-0.5)*1000;	
			p.y()=(drand48()-0.5)*1000;	
			p.z()=-800;
		}
 };


void GameIntro::Key(unsigned char key)
{
	switch(key){
		case ' ' : Game=&GS;
			GS.Init();
			GS.Reshape(ScreenW,ScreenH);
			break;
		}
}

void GameIntro::Init(int,int)
{
	Ast3DFract::SetSharedTexture("data/ast.png");
	Mesh AstM;
	AstM.Open("data/tetra.raw");
	if(A[0].size()<AstNum) {
		for(int j=0;j<3;j++){
			A[j].resize(AstNum);
			for(int i=0;i<AstNum;i++)
				{
					A[j][i]=new Ast3DFract(AstM,Ast::LargeSize,j+1,i+1);
					if(j==0){
					srand(((i+3771)*346793171)%1773721);
					A[j][i]->p.x()=(drand48()-0.5)*1000;	
					A[j][i]->p.y()=(drand48()-0.5)*1000;	
					A[j][i]->p.z()=-800;
					A[j][i]->v.z()=200;
					A[j][i]->av=20+drand48()*70;
					A[j][i]->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
					}
				}
		}
	}
	float lp[4]={1,1,3,0};
	glLightfv(GL_LIGHT0,GL_POSITION,lp);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	//glEnable(GL_CULL_FACE);
	// Remove Comments if some face disappears...
	glDisable(GL_CULL_FACE);
	//glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);

	/* play the module file */  
  if(AudioFlag) {
		APlayModule(lpModule);
		ASetModuleVolume(8);
	}
}

int GameIntro::Draw()
{ static float angle=0;
	int nt=0;
	glLoadIdentity();
	glPushMatrix();
	glColor3f(.7,.7,1);
	int sw=342; // subtitle width
	glPrintf(ScreenW/2,ScreenH/3,1,12, TitleMessage);
	angle+=5.0;
	glRotatef(angle,0,1,0);
	glScalef(40,40,40);
	nt+=Title.DrawDL();
	glPopMatrix();
	glPushMatrix();
	glTranslatef(0,0,-1000);
	glScalef(2000,2000,2000);
	s.Draw();
  glPopMatrix();

	int j=0;
	for(int i=0;i<AstNum;i++)
		{
			j=0;
			if(A[j][i]->p.z()>-200) j=1;
			if(A[j][i]->p.z()>00) j=2;
			nt+=A[j][i]->Draw();
		}
 return nt;
}

void  GameIntro::Reshape(int w,int h)
{
  ScreenW=w;
  ScreenH=h;
	int SpaceSize=100;
	float wr=float(w)/h;
  float hr=float(h)/w;

	if(w>h) hr=1;
	else wr=1;
  int VL = -SpaceSize*wr;
  int VR =  SpaceSize*wr;
  int VB = -SpaceSize*hr;
  int VT =  SpaceSize*hr;
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

	gluPerspective(90,1,5,1201);
  gluLookAt(0,0,200,0,0,0,0,1,0);
	glMatrixMode(GL_MODELVIEW);
}

void GameIntro::Update(int dt)
{
	static int t=0;
	static int titind;
	t+=dt;
	if(t>2000) 
		{
			t=0;
			titind=(titind+1)%3;
			TitleMessage=message[titind];
		}
	s.Update(dt);
		for(int i=0;i<AstNum;i++)
		{
			 A[0][i]->Update(dt);
			 Clip(A[0][i]->p);
		}
		for(int j=1;j<3;j++)
			for(int i=0;i<AstNum;i++){
				A[j][i]->angle=A[0][i]->angle;
				A[j][i]->p=A[0][i]->p;
			}
	
}